#!/usr/bin/ruby

@base_dir = Dir::getwd + "/" + File.dirname($0)

@tarballs = [
  "PA10_owa",
  "arm_moving_pid_pa10_module",
  "senario_executive_module",
  "force_pos_hybrid_module",
  "port_splitter",
  "redundancy_minimize_norm"
]

def exec(cmd)
  puts "<execute> #{cmd}"
  system(cmd)
end

def exec_async(cmd)
  puts "<execute async> #{cmd}"
  system(cmd + " &")
end

def extract
  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    system("tar xvfz #{f}*.tar.gz")
  }
end

def make
  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    Dir::chdir(f)
    if File.exist? "Makefile"
      system("make")
    end
  }
end
 
def clean
  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    Dir::chdir(f)
    if File.exist? "Makefile"
      system("make clean")
    end
  }
end

def tarball
  clean

  datestr = Time.now.strftime("%Y%m%d")

  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    Dir::chdir(f)
    if File.exist? "Makefile"
      system("make clean")
    end

    Dir::chdir(@base_dir)

    cmd = "tar cvfz #{f}-#{datestr}.tar.gz --exclude=.svn #{f}"
    puts cmd
    system(cmd)

  }

  Dir::chdir(@base_dir)
  tmpdir = "/tmp/force_pos_hybrid_module-#{datestr}"
  exec "mkdir -p #{tmpdir}"
  exec "mv *.tar.gz #{tmpdir}"
  exec "cp README.txt #{tmpdir}"
  exec "cp Makefile #{tmpdir}"
  exec "cp build.rb #{tmpdir}"
  exec "cp hybrid-demo.xml #{tmpdir}"

  Dir::chdir(tmpdir)
  Dir::chdir("..")
  exec "tar cvfz force_pos_hybrid_module-#{datestr}.tar.gz force_pos_hybrid_module-#{datestr}" 
  exec "mv force_pos_hybrid_module-#{datestr}.tar.gz ~"
  exec "rm -rf #{tmpdir}"
end

#
#
#
@activate_modules = [
  ["./arm_moving_pid_pa10_module/", "ArmMovingPIDPA10ModuleComp"],
  ["./redundancy_minimize_norm/",   "RedundancyMinimizeNormComp"],
  ["./port_splitter/",              "PortSplitterComp"],
  ["./force_pos_hybrid_module/",    "ForcePosHybridComp"],
  ["./senario_executive_module/",   "SenarioExecutiveModuleComp"],
]

def _setup_modules
  p @activate_modules
  @activate_modules.each {|a|
    Dir.chdir(@base_dir)
    Dir.chdir(a[0])
    exec_async("make run")
  }
  Dir.chdir(@base_dir)
end

@amm = "ArmMovingPIDPA10Module"
@rmm = "RedundancyMinimizeNorm"
@psm = "PortSplitter"
@sem = "SenarioExecutiveModule"
@hbm = "ForcePosHybrid"

def _connect_module()

  conn = [
    # RedundancyMinimizeNorm
    [@amm, "q_cur_out",  @rmm, "q_cur_in"],
    [@amm, "q_ref_in",   @rmm, "q_ref_out"],
    [@rmm, "xyzrpy_out", @psm, "in"],
    [@psm, "out0",       @sem, "in"],

    # ForcePosHybrid in
    [@psm, "out1",       @hbm, "X_k_1"],
    [@sem, "Xd_k",       @hbm, "Xd_k"],
    [@sem, "Fd",         @hbm, "Fd"],
    [@sem, "diag_S",     @hbm, "diag_S"],
    [@amm, "FT_out",     @hbm, "F"],

    # ForcePosHybrid out
    [@hbm, "X_k",        @rmm, "xyzrpy_in"],
  ]

  ns = "/127.0.0.1:2809/"

  conn.each {|a|
    cmd = "rtcon #{ns}#{a[0]}0.rtc:.#{a[1]} #{ns}#{a[2]}0.rtc:.#{a[3]}"
    exec(cmd)
  }
end

def _activate_module(rtc)
  cmd = "rtact /localhost:2809/" + rtc
  exec(cmd)
end

def _set_config(rtc, key, val)
  cmd = "rtconf /localhost:2809/#{rtc} set #{key} #{val}"
  exec(cmd)
end

def setup
  _setup_modules
  sleep 3
  _connect_module()
  sleep 1
  _set_config(@rmm + "0.rtc", "str_link_base",  "BASE_JOINT")
  _set_config(@rmm + "0.rtc", "str_link_end",   "PA10_JOINT7")
  _set_config(@rmm + "0.rtc", "str_model_file", "../PA10_owa/pa10.wrl")
  sleep 1
  _activate_module(@rmm + "0.rtc")
  _activate_module(@hbm + "0.rtc")
  _activate_module(@psm + "0.rtc")
end

def shutdown
  @activate_modules.each{|a|
    cmd = "rtexit /localhost:2809/" + a[1].gsub(/Comp/,"") + "0.rtc";
    exec(cmd)
  }
end


def usage
  puts "usage : build.sh {extract|make|run|tarball|cleean}"
  exit 0
end
 
if __FILE__ == $0
  usage if ARGV.size != 1

  puts "execute cmd=#{ARGV[0]}"
  puts "base_dir = #{@base_dir}"
  Dir::chdir(@base_dir)

  send(ARGV[0])
end
